from abc import ABCMeta, abstractmethod
from sqlalchemy import create_engine
from sqlalchemy.orm import declarative_base, sessionmaker
from typing import List
from platformdirs import user_data_dir

db_uri = f"{user_data_dir('yawgbot', 'rcastellotti')}/yawgbot.sqlite"


class PluginBase(metaclass=ABCMeta):
    """Base class for plugins"""

    engine = create_engine(f"sqlite:///{db_uri}", echo=False)
    Base = declarative_base()
    Session = sessionmaker(bind=engine)

    @abstractmethod
    def contact_ad(self) -> None:
        """this method should contact the ad"""

    @abstractmethod
    def get_ads(self, url) -> List[str]:
        """this method should scrape a platform and return the HTML containing all the ads"""

    @abstractmethod
    def create_listing(self, ad) -> None:
        """this method should extract relevant information from an ad (an element of the list returned by getAds)"""

    @abstractmethod
    def run(self):
        """this method runs the plugin"""

    # metodo che notifica
    # inventarsi qualcosa per assegnare alla piattaforma un colore ``
    # ogni plugin deve occuparsi di definire la condizione "ho trovato una immagine buona"
    # voglio che tutto sia typato
    # cambiare gli hover dei bottoni giu in basso



from yawgbot.bot import Listing
from flask import request
from sqlalchemy import create_engine
from sqlalchemy.orm import declarative_base, sessionmaker, scoped_session
import random
import os
from flask import Flask, render_template
from platformdirs import user_data_dir

db_uri = f"{user_data_dir('yawgbot', 'rcastellotti')}/yawgbot.sqlite"

engine = create_engine(f"sqlite:///{db_uri}", echo=False)
Base = declarative_base()
session_factory = sessionmaker(bind=engine)
Session = scoped_session(session_factory)

app = Flask(__name__, template_folder=".")


def random_class():
    wes = [
        "Royal2q0",
        "Royal2q1",
        "Royal2q2",
        "Royal2q3",
        "Royal2q4",
        "Zissou1q1",
        "GrandBudapest2q3",
        "BottleRocket2q1",
        "GrandBudapest1q3",
        "IsleofDogs1q0",
    ]
    return random.choice(wes)


@app.get("/")
def index():
    page = request.args.get("page")
    if page is None:
        page = 0
        next_page = 1
    page = int(page)
    next_page = page + 1
    prev_page = page - 1
    session = Session()
    listings = (
        session.query(Listing)
        .order_by(Listing.added_at.desc())
        .filter(Listing.price.isnot(""))
        .limit(20)
        .offset(page * 20)
        .all()
    )
    Session.remove()
    return render_template(
        "index.html",
        listings=listings,
        random_class=random_class,
        prev_page=prev_page,
        next_page=next_page,
    )


def run():
    app.run(host="0.0.0.0")


if __name__ == "__main__":
    app.run(host="0.0.0.0")



from yawgbot import Bot

from celery import Celery

CELERY_RESULT_DBURI = "db+sqlite:///celerydb.sqlite"
BROKER_URL = "sqla+sqlite:///celerydb.sqlite"

celery = Celery("main", broker=BROKER_URL, backend=CELERY_RESULT_DBURI)


@celery.task
def runBot():
    bot = Bot(platforms=["wg-gesucht"])
    bot.run()


@celery.on_after_configure.connect
def setup_periodic_tasks(sender, **kwargs):
    sender.add_periodic_task(60 * 5, runBot.s(), name="run yawgbot every 5 minutes")



import requests
from bs4 import BeautifulSoup as bs
import logging
from yawgbot.pluginBase import PluginBase
from yawgbot.bot import Listing
import re
import os
import json


class YawgbotPlugin(PluginBase):
    ID_REGEX = r"\.(\d*)\.html"
    IMG_REGEX = r"\(([^\)]+)\)"
    WG_GESUCHT_USERNAME = os.environ["WG_GESUCHT_USERNAME"]
    WG_GESUCHT_PASSWORD = os.environ["WG_GESUCHT_PASSWORD"]
    WG_GESUCHT_TEMPLATE_MESSAGE = os.environ["WG_GESUCHT_TEMPLATE_MESSAGE"]
    COLOR = "#f97316"
    PLATFORM = "https://www.wg-gesucht.de/"
    BASE_URL = "https://www.wg-gesucht.de/wg-zimmer-und-1-zimmer-wohnungen-in-Munchen.90.0+1.1.{}.html"

    def __init__(self):
        self.s = requests.Session()
        r = self.s.post(
            "https://www.wg-gesucht.de/ajax/sessions.php?action=login",
            json={
                "login_email_username": self.WG_GESUCHT_USERNAME,
                "login_password": self.WG_GESUCHT_PASSWORD,
            },
        )
        self.csrf_token = json.loads(r.text)["csrf_token"]
        self.user_id = json.loads(r.text)["user_id"]
        logging.debug(r.text)

    def get_ads(self, url):
        r = requests.get(url)
        soup = bs(r.text, "html.parser")
        logging.debug(soup.prettify())
        ads = soup.select(".wgg_card:not(.noprint)")
        return ads

    def create_listing(self, ad):
        url = "https://www.wg-gesucht.de" + ad.find_all("a")[1]["href"]
        name = ad.find_all("a")[1].text.strip()
        slug = re.findall(self.ID_REGEX, url)[0]
        size = ad.find_all(attrs={"class": "col-xs-3"})[1].text.strip()
        dates = "".join(ad.find(attrs={"class": "col-xs-5 text-center"}).text.split())
        price = ad.find_all(attrs={"class": "col-xs-3"})[0].text.strip()
        location_string = (
            ad.find_all(attrs={"class": "col-xs-11"})[0]
            .text.replace("\n", "")
            .replace(" ", "")
        )
        location = location_string[location_string.find("|") + 1 :].replace("|", " | ")
        image = re.findall(
            self.IMG_REGEX, str(ad.find(attrs={"class": "card_image"}).find("a"))
        )[0]

        listing = Listing(
            name=name,
            url=url,
            slug=slug,
            size=size,
            location=location,
            image=image,
            dates=dates,
            price=price,
            platform=self.PLATFORM,
            color=self.COLOR,
        )
        session = self.Session()

        if not session.query(Listing).filter_by(slug=listing.slug).first():
            logging.info(f"new ad found: {listing.name}")
            session.add(listing)
            self.contact_ad(listing.slug)
            session.commit()
        else:
            logging.info(f"skipping ad:{listing.name}")

    def contact_ad(self, slug):

        json_data = {
            "user_id": self.user_id,
            "csrf_token": self.csrf_token,
            "messages": [
                {
                    "content": self.WG_GESUCHT_TEMPLATE_MESSAGE,
                    "message_type": "text",
                },
            ],
            "ad_type": "0",
            "ad_id": slug,
        }

        r = self.s.post(
            "https://www.wg-gesucht.de/ajax/conversations.php",
            params={"action": "conversations"},
            json=json_data,
        )
        logging.debug(r.text)
        pass

    def run(self):
        for i in range(5):
            ads = self.get_ads(self.BASE_URL.format(i))
            logging.debug(ads)
            for ad in ads:
                self.create_listing(ad)



import os
from dotenv import load_dotenv
from sqlalchemy import create_engine, Column, Integer, String, DateTime
from sqlalchemy.orm import declarative_base
import datetime
from sqlalchemy_utils import database_exists
import importlib
from yawgbot.pluginBase import PluginBase
from platformdirs import user_data_dir

load_dotenv()
db_uri = f"{user_data_dir('yawgbot', 'rcastellotti')}/yawgbot.sqlite"
engine = create_engine(f"sqlite:///{db_uri}", echo=False)
Base = declarative_base()


class Listing(Base):
    __tablename__ = "listings"
    id = Column(Integer, primary_key=True)
    slug = Column(Integer, nullable=False)
    added_at = Column(DateTime, default=datetime.datetime.now)
    price = Column(Integer)
    name = Column(String)
    url = Column(String)
    image = Column(String)
    location = Column(String)
    size = Column(String)
    dates = Column(String)
    color = Column(String)
    platform = Column(String)


class Bot:
    TG_API_KEY = os.environ["TG_API_KEY"]
    TG_CHAT_ID = os.environ["TG_CHAT_ID"]

    def __init__(self, platforms: list = []):
        self._platforms = []
        if platforms:
            for plugin in platforms:
                plugin_to_add = importlib.import_module(
                    f"yawgbot.plugins.{plugin}"
                ).YawgbotPlugin()
                if isinstance(plugin_to_add, PluginBase):
                    self._platforms.append(plugin_to_add)

        os.makedirs(user_data_dir("yawgbot", "rcastellotti"), exist_ok=True)
        if not database_exists(f"sqlite:///{db_uri}"):
            Base.metadata.create_all(engine)

    def run(self):
        for platform in self._platforms:
            platform.run()
